﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using DB_ABT.Models;
using DB_ABT.ViewModels;
using DB_ABT.DBS_Klassen;

namespace DB_ABT.Fenster {
    /// <summary>
    /// Interaktionslogik für FReadInputMitarbeiter.xaml
    /// </summary>
    public partial class FReadInputMitarbeiter : Window {

        private bool retcode = false;
        private ModelEmployee mit = null;
        private double monthsalary=0.0;
        private long aindex = 0;
        private ObservableCollection_Dept listAbt;


        public bool Retcode {
            get { return retcode; }
        }


        public FReadInputMitarbeiter(Window owner, ModelEmployee mit) {
            InitializeComponent();
            this.Owner = owner;  // Important, because CenterOwner           
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            listAbt = Dbs_QueriesExceute.getItemsFromDept();
            cbAbteilung.Items.Clear();
            foreach (ModelDept abt in listAbt.Items) {
                cbAbteilung.Items.Add(abt);
            }
            cbAbteilung.SelectedIndex = -1;
            if (mit != null) {
                tFirstname.Text = mit.Firstname;
                tLastname.Text = mit.Lastname;
                tMonatsgehalt.Text = mit.Monthsalary.ToString("##0.##");  // liefert 123,55  !!
                for (int i = 0; i < listAbt.Items.Count; i++) {
                    ModelDept abt2 = listAbt.Items[i];
                    if (abt2.PIndex == mit.DeptIindex) {
                        cbAbteilung.SelectedIndex = i;
                        break;
                    }
                }
            }
        }

        public String Firstname {
            get { return tFirstname.Text.Trim(); }
        }
        public String Lastname {
            get { return tLastname.Text.Trim(); }
        }
        public double Monthsalary {
            get { return monthsalary; }
        }

        public long AIndex {
            get { return aindex; }
        }


        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (tFirstname.Text.Trim().Length == 0) {
                Basis.ErrorMsg("Geben Sie bitte einen Vornamen ein", "Hinweis");
                tFirstname.Focus();
                return;
            }
            if (tLastname.Text.Trim().Length == 0) {
                Basis.ErrorMsg("Geben Sie bitte einen Nachnamen ein", "Hinweis");
                tLastname.Focus();
                return;
            }
            String s = tMonatsgehalt.Text.Trim().Replace(",",".");
            if (!double.TryParse(s, System.Globalization.NumberStyles.AllowDecimalPoint, 
                           System.Globalization.CultureInfo.InvariantCulture,  out monthsalary)) {
                Basis.ErrorMsg("Geben Sie bitte ein numerisches Monatsgehalt ein", "Hinweis");
                tMonatsgehalt.Focus();
                return;
            }
            if (monthsalary<0.0) {
                Basis.ErrorMsg("Geben Sie bitte ein gültiges Monatsgehalt ein", "Hinweis");
                tMonatsgehalt.Focus();
                return;
            }
            if (cbAbteilung.SelectedIndex < 0) {
                Basis.ErrorMsg("Geben wählen Sie eine Abteilung aus", "Hinweis");
                cbAbteilung.Focus();
                return;
            }
            aindex = listAbt.Items[cbAbteilung.SelectedIndex].PIndex;

            retcode = true;
            Close();
        }

        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }

        private void Window_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Return)) {
                bnOk_Click(sender, e);
            }
        }
    }
}
