﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Win32;    // OpenDialog
using System.Data;
using System.Diagnostics;   // DataTable
using System.IO;

using System.Collections.ObjectModel;
//using DB_ABT.Fenster;
using DB_ABT.DBS_Klassen;
using DB_ABT.Middleware;
using DB_ABT.Fenster;




namespace DB_ABT {
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window {


        private String databasefile = null;
        private String registryfilename = "";

        public MainWindow() {
            InitializeComponent();
        }


        private void Window_Loaded(object sender, RoutedEventArgs e) {
            setGUI();
            registryfilename = Basis.ReadRegString(MyRegistry.C_REG_ADR_PRJ, MyRegistry.C_REG_PRJNAME);
            if (!registryfilename.Equals("") && File.Exists(registryfilename)) {
                bnOpenProjekt.Content = "ABT: " + Basis.getFilename(registryfilename);
                bnOpenProjekt.ToolTip = "Öffnen des alten Projektes: (" + registryfilename + ")";
            }
            else {
                bnOpenProjekt.IsEnabled = false;
            }
        }

        private void setGUI() {
            CommandBindings.Add(new CommandBinding(MyCommands.NewDBFile, NewDBFileExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.OpenDBFile, OpenDBFileExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Close, CloseExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Abt, AbtExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Employee, MitarbeiterExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Employee2, Mitarbeiter2Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.MasterDetails, MasterDetailsExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.DBS_Console, DBS_ConsoleExecuted));
        }


        private void bnOpenProjektes_Click(object sender, RoutedEventArgs e) {
            if (!registryfilename.Equals("")) {
                openDataBase(registryfilename);
            }
        }


        private void OpenDBFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Datenbanken (*.fdb)|*.fdb;*.dgb|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".fdb";
            openFileDialog1.InitialDirectory = "";
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true) {
                openDataBase(openFileDialog1.FileName);
            }
        }

        private void openDataBase(String filename) {
            databasefile = filename;
            Basis.WriteRegString(MyRegistry.C_REG_ADR_PRJ, MyRegistry.C_REG_PRJNAME, filename);

            bnOpenProjekt.Content = "Prj: " + Basis.getFilename(filename);
            bnOpenProjekt.IsEnabled = false;

            this.Title = "Prj: " + Basis.getFilename(filename);
            LProjekt.Content = filename;
            Dbs dbs = Dbs.getDbsInstance();
            dbs.Close();
            dbs.setDbs(filename);
            bnOpenProjekt.Foreground = Brushes.Blue;
        }

        private void DBS_ConsoleExecuted(object sender, ExecutedRoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "FBConsole.exe";
            // Prog.StartInfo.Arguments = "C:\\Daten\test.txt";
            Prog.Start();
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
            // e.Cancel = true;  // kein ende
            e.Cancel = false;
            Basis.WriteRegInt(MyRegistry.C_REG_ADR_PRJ, "iTop1", (int)this.Top);
            Basis.WriteRegInt(MyRegistry.C_REG_ADR_PRJ, "iLeft1", (int)this.Left);
            Basis.WriteRegInt(MyRegistry.C_REG_ADR_PRJ, "iWidth1", (int)this.Width);
            Basis.WriteRegInt(MyRegistry.C_REG_ADR_PRJ, "iHeight1", (int)this.Height);
        }

        private void CloseExecuted(object sender, ExecutedRoutedEventArgs e) {
            Close();
        }

        private void CheckCanExecuted(object sender, CanExecuteRoutedEventArgs e) {
            e.CanExecute = databasefile != null;
        }

        private void NewDBFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            const String sqlfilename = "wpf_abt.txt";
            if (File.Exists(sqlfilename)) {
                string filename = "";
                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Filter = "Datenbankdateien (*.fdb)|*.fdb|Alle Dateien (*.*)|*.*";
                saveFileDialog1.FilterIndex = 1;// zählt von 1 !
                saveFileDialog1.DefaultExt = ".fdb";
                saveFileDialog1.InitialDirectory = Environment.CurrentDirectory;
                saveFileDialog1.RestoreDirectory = true;
                if (saveFileDialog1.ShowDialog() == true) {
                    filename = saveFileDialog1.FileName.ToString();
                    Dbs dbs = Dbs.getDbsInstance();
                    dbs.createDatabase(filename);
                    dbs.setDbs(filename);
                    FProgressBarCreateTables dialog = new FProgressBarCreateTables(this, sqlfilename);
                    dialog.ShowDialog();
                    // dbs_createCommands.insertTables(this);
                    openDataBase(filename);
                }
            }
            else {
                Basis.ErrorMsg("Die SQL-Datei ist nicht vorhanden\r\n" + sqlfilename, "Erzeugen einer neuen Datenbank");
            }
        }

        private void AbtExecuted(object sender, ExecutedRoutedEventArgs e) {
            Mid_Dept.ShowAbt(this, "Abteilung", "Abteilung");
        }

        private void MitarbeiterExecuted(object sender, ExecutedRoutedEventArgs e) {
            Mid_Employee.ShowMitarbeiter(this);
        }
        private void Mitarbeiter2Executed(object sender, ExecutedRoutedEventArgs e) {
            Mid_Employee.ShowMitarbeiter2(this);
        }
        private void MasterDetailsExecuted(object sender, ExecutedRoutedEventArgs e) {
            Mid_Employee.ShowMasterDetails(this);
        }

        

    }
}
