﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized; // ICollectionView
using DB_ABT.UserControls;
using DB_ABT.DBS_Klassen;
using DB_ABT.ViewModels;
using DB_ABT.Models;
using DB_ABT.Fenster;

//  Hinweis
// im datagrid wurden drei Spalten eingetragen NUR damit klappt das Anzeigen bei einer Änderung
// damit nun aber nicht zwei Varianten der Spalten eingetragen werden, gibt es
// die Eigenschaft : AutoGenerateColumns="False"

namespace DB_ABT.Views {
    /// <summary>
    /// Interaktionslogik für FAbteilung.xaml
    /// </summary>
    public partial class FAbteilung : Window, INavigatorbar, IError, INotifyPropertyChanged {

        ObservableCollection<ModelDept> abtitems;
        ObservableCollection_Dept abt_class;
        String filter = "";

     
        private void NotifyPropertyChanged(string p) {
            if (PropertyChanged != null) {
                PropertyChanged(datagrid, new PropertyChangedEventArgs(p));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;


        public FAbteilung(Window owner, String abteilung, ObservableCollection_Dept abt_class) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            LAbteilung.Content = "Abt: " + abteilung;
            this.abt_class = abt_class;
            this.abtitems = abt_class.Items;
            datagrid.ItemsSource = abt_class.Items;  // one way

            datagrid.RowValidationRules.Add(new AbteilungValidationRule1(this, datagrid, abtitems) {
                ValidationStep = ValidationStep.UpdatedValue
            });            
        }


      


        private void Window_Loaded(object sender, RoutedEventArgs e) {
            navibar.setDatabaseParent(this,1);
            int n = datagrid.Columns.Count;
            navibar.setCount(abtitems.Count);
            Title = "Anzahl: " + abtitems.Count.ToString();

            datagrid.Columns[0].Width = 100;
            datagrid.Columns[2].Width = 50;
            datagrid.Columns[1].Width = 300; // datagrid.Width - 200;


        }



        // wird vom navibar aufgerufen
        public void setError(bool error) {
            bnOk.IsEnabled = !error;
        }

        public void setFirst(int nr) {
            datagrid.SelectedIndex = 0;
            datagrid.ScrollIntoView(datagrid.SelectedItem);
            datagrid.Focus();
        }

        public void setPrev(int nr) {
            if (datagrid.SelectedIndex > 0) {
                datagrid.SelectedIndex = datagrid.SelectedIndex - 1;
                //datagrid.SelectedItem = abtitems[datagrid.SelectedIndex - 1];
                //datagrid.UpdateLayout();
                datagrid.ScrollIntoView(datagrid.SelectedItem);
                datagrid.Focus();
            }
        }

        public void setNext(int nr) {
            if (datagrid.SelectedIndex < (abtitems.Count - 1)) {
                datagrid.SelectedIndex = datagrid.SelectedIndex + 1;
                //datagrid.SelectedItem = abtitems[datagrid.SelectedIndex + 1];
                //datagrid.UpdateLayout();
                datagrid.ScrollIntoView(datagrid.SelectedItem);
                datagrid.Focus();
            }
        }

        public void setLast(int nr) {
            datagrid.SelectedIndex = abtitems.Count - 1;
            datagrid.ScrollIntoView(datagrid.SelectedItem);
            datagrid.Focus();
        }


        public void setInsert(int nr) {
            String kurzBez = "";
            String bez = "";
            bool error = true;
            while (error) {
                error = true;
                FReadInput2 dialog = new FReadInput2(this, "Kurz-Bezeichnung", kurzBez, "Bezeichnung", bez);
                dialog.ShowDialog();
                if (!dialog.Retcode) return;
                kurzBez = dialog.Input1;
                bez = dialog.Input2;
                // test auf doppelte Items
                String abtUpper = kurzBez.ToUpper();
                if (abtUpper.Length == 0) {
                    MessageBox.Show("Bitte geben Sie eine Kurzbezeichnung ein. Eine leere Kurzbezeichnung ist verboten.", "Eintrag Bearbeitung", MessageBoxButton.OK, MessageBoxImage.Error);
                }
                else {

                    for (int i = 0; i < abtitems.Count; i++) {
                        String sk = abtitems[i].ShortName.ToUpper();
                        if (abtitems[i].ShortName.ToUpper().Equals(abtUpper)) {
                            MessageBox.Show("Der Bezeichnung ist schon vorhanden.", "Neuer Eintrag", MessageBoxButton.OK, MessageBoxImage.Error);
                            continue;
                        }
                    }
                    error = false;
                    Dbs dbs = Dbs.getDbsInstance();
                    long pindex = Dbs.getSequenz("seq");
                    ModelDept abt = new ModelDept(pindex, kurzBez, bez);
                    abtitems.Add(abt);
                }
            }
        }

        public void setDelete(int nr) {
            ModelDept abt = (ModelDept)datagrid.SelectedItem;
            if (abt != null) {
                // check, ob die Abteilung noch benutzt wird?
                bool gefunden = Dbs_QueriesExceute.isForeignKeyInMitarbeiter(abt.PIndex);
                if (gefunden) {
                    Basis.Message("Diese Abteilung darf nicht gelöscht werden, da sie noch verwendet wird", "Hinweis");
                }
                else {
                    if (Basis.readYesNo("Wollen Sie wirklich diese Abteilung löschen ?\r\n" +
                        "Abteilung: " + abt.ShortName + " (" + abt.LongName + ")", "Abteilung löschen")) {
                        abt_class.Delete(abt);
                        navibar.setCount(abtitems.Count);
                        // setFilter(filter);
                    }
                }
            }
        }

        public void setCancel(int nr) {
            ((DataRowView)datagrid.SelectedItem).Row.CancelEdit();
        }

        public void setFilter(int nr, String filter) {
            this.filter = filter.Trim();
            if (filter.Length == 0) {
                //datagrid.ItemsSource = abtitems;
                ICollectionView cv = CollectionViewSource.GetDefaultView(datagrid.ItemsSource);
                cv.Filter = null;
            }
            else {
                ICollectionView cv = CollectionViewSource.GetDefaultView(datagrid.ItemsSource);
                cv.Filter = abt => (abt as ModelDept).ShortName.StartsWith(filter);
                //var fc = from abt in abtitems
                //         where abt.KurzBez.Contains(filter)
                //         select abt;
                //datagrid.ItemsSource = fc;
            }
            navibar.setCount(datagrid.Items.Count);
            // MyObservableCollection<Katalog> FilteredData = new MyObservableCollection<Katalog>(fc);           
        }

        private bool getModify() {
            bool modify = false;
            foreach (ModelDept abt in abtitems) {
                if (abt.modifyAttrib) modify = true;
                if (abt.newAttrib) modify = true;
            }
            if (abt_class.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: abt_class.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                abt_class.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }


        public class AbteilungValidationRule1 : ValidationRule {

            private DataGrid grid;
            private ObservableCollection<ModelDept> abtitems;
            private IError error;

            public AbteilungValidationRule1(IError error, DataGrid grid, ObservableCollection<ModelDept> abtitems) {
                this.error = error;
                this.grid = grid;
                this.abtitems = abtitems;
            }

            public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo) {
                ModelDept abt = (value as BindingGroup).Items[0] as ModelDept;
                int index = grid.Items.IndexOf(abt);

                String abtUpper = abt.ShortName.Trim().ToUpper();
                if (abtUpper.Length == 0) {
                    MessageBox.Show("Bitte geben Sie einen Kurzbezeichung ein. Eine leere Kurzbezeichung ist verboten.", "Eintrag Bearbeitung", MessageBoxButton.OK, MessageBoxImage.Error);
                    error.setError(true);
                    return new ValidationResult(false, "Bitte geben Sie eine Kurzbezeichung ein. Eine leerer Kurzbezeichung ist verboten.");
                }
                for (int i = 0; i < abtitems.Count; i++) {
                    String sk = abtitems[i].ShortName.ToUpper();
                    if (i != index && abtitems[i].ShortName.ToUpper().Equals(abtUpper)) {
                        MessageBox.Show("Die Kurzbezeichung ist schon vorhanden.", "Eintrag Bearbeitung", MessageBoxButton.OK, MessageBoxImage.Error);
                        error.setError(true);
                        return new ValidationResult(false, "Die Kurzbezeichung ist schon vorhanden.");
                    }
                }
                error.setError(false);
                return ValidationResult.ValidResult;
            }
        } // AbteilungValidationRule1



        private void datagrid_BeginningEdit(object sender, DataGridBeginningEditEventArgs e) {
            DataGridColumn col = e.Column;
            if (col.Header.Equals("PIndex")) {  // Pindex
                e.Cancel = true;
            }
        }

        private void datagrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e) {
            DataGridColumn col = e.Column;
            e.Cancel = false;
            if (col.Header.Equals("KurzBez")) {
                TextBox t = e.EditingElement as TextBox;
                DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;

            }
            if (col.Header.Equals("Bez")) {
                TextBox t = e.EditingElement as TextBox;
                DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;
            }

        }


        private void datagrid_RowEditEnding(object sender, DataGridRowEditEndingEventArgs e) {

        }



    }
}
