﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized; // ICollectionView
using DB_ABT.UserControls;
using DB_ABT.DBS_Klassen;
using DB_ABT.ViewModels;
using DB_ABT.Models;
using DB_ABT.Fenster;

//  Hinweis
// im datagrid wurden drei Spalten eingetragen NUR damit klappt das Anzeigen bei einer Änderung
// damit nun aber nicht zwei Varianten der Spalten eingetragen werden, gibt es
// die Eigenschaft : AutoGenerateColumns="False"


namespace DB_ABT.Views {
    /// <summary>
    /// Interaktionslogik für FMitarbeiter.xaml
    /// </summary>
    public partial class FEmployee : Window, INavigatorbar, IError {

        ObservableCollection<ModelEmployee> empl_items;
        ObservableCollection_Employee class_Employee;
        ICollectionView collView = null;



        public FEmployee(Window owner, ObservableCollection_Employee mit_class) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            this.class_Employee = mit_class;
            this.empl_items = mit_class.Items;
            
        }


        private void Window_Loaded(object sender, RoutedEventArgs e) {
            navibar.setDatabaseParent(this, 1);

            datagrid.ItemsSource = class_Employee.Items;  // one way
            collView = CollectionViewSource.GetDefaultView(datagrid.ItemsSource);
            collView.SortDescriptions.Add(new SortDescription("Lastname", ListSortDirection.Ascending));
            collView.SortDescriptions.Add(new SortDescription("Firstname", ListSortDirection.Ascending));

            int n = datagrid.Columns.Count;
            navibar.setCount(empl_items.Count);
            Title = "Employee-Dialog";
            LMitarbeiter.Content = "Employee: ";

            datagrid.Columns[0].Width = 100;
            datagrid.Columns[1].Width = 200; // datagrid.Width - 200;
            datagrid.Columns[2].Width = 50;

            datagrid.CanUserAddRows = false;
        }



        // wird vom navibar aufgerufen
        public void setError(bool error) {
            bnOk.IsEnabled = !error;
        }

        public void setFirst(int nr) {
            datagrid.SelectedIndex = 0;
            datagrid.ScrollIntoView(datagrid.SelectedItem);
            datagrid.Focus();
        }

        public void setPrev(int nr) {
            if (datagrid.SelectedIndex > 0) {
                datagrid.SelectedItem = empl_items[datagrid.SelectedIndex - 1];
                datagrid.UpdateLayout();
                datagrid.ScrollIntoView(datagrid.SelectedItem);
                datagrid.Focus();
            }
        }

        public void setNext(int nr) {
            if (datagrid.SelectedIndex < (empl_items.Count - 1)) {
                datagrid.SelectedItem = empl_items[datagrid.SelectedIndex + 1];
                datagrid.UpdateLayout();
                datagrid.ScrollIntoView(datagrid.SelectedItem);
                datagrid.Focus();
            }
        }

        public void setLast(int nr) {
            datagrid.SelectedIndex = empl_items.Count - 1;
            datagrid.ScrollIntoView(datagrid.SelectedItem);
            datagrid.Focus();
        }


        public void setInsert(int nr) {
            ModelEmployee mit = new ModelEmployee();            
            FReadInputMitarbeiter dialog = new FReadInputMitarbeiter(this, mit);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                mit.Firstname = dialog.Firstname;
                mit.Lastname = dialog.Lastname;
                mit.Monthsalary = dialog.Monthsalary;
                mit.DeptIindex = dialog.AIndex;
                Dbs dbs = Dbs.getDbsInstance();
                mit.PIndex = Dbs.getSequenz("seq");
                empl_items.Add(mit);
                navibar.setCount(empl_items.Count);
            }
        }

        public void setDelete(int nr) {
            ModelEmployee empl = (ModelEmployee)datagrid.SelectedItem;
            if (empl != null) {
                if (Basis.readYesNo("Wollen Sie wirklich diesen Mitarbeiter löschen ?\r\n" +
                        "Mitarbeiter: " + empl.Lastname + ", " + empl.Firstname + ")", "Mitarbeiter löschen")) {
                    class_Employee.Delete(empl);
                    navibar.setCount(empl_items.Count);
                }
            }
        }

        public void setCancel(int nr) {
            ((DataRowView)datagrid.SelectedItem).Row.CancelEdit();
        }

        public void setFilter(int nr,String filter) {
            String f = filter.Trim();
            if (f.Length == 0) {                
                collView.Filter = null;
            }
            else {                
                collView.Filter = mit => (mit as ModelEmployee).Lastname.StartsWith(filter);
            }
            navibar.setCount(datagrid.Items.Count);
        }

        private bool getModify() {
            bool modify = false;
            foreach (ModelEmployee mit in empl_items) {
                if (mit.modifyAttrib) modify = true;
                if (mit.newAttrib) modify = true;
            }
            if (class_Employee.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: class_Employee.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                class_Employee.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }


       

        // Spalten, die nicht editierbar sein sollen
        private void datagrid_BeginningEdit(object sender, DataGridBeginningEditEventArgs e) {
            DataGridColumn col = e.Column;
            if (col.Header.Equals("PIndex") || col.Header.Equals("AIndex") || col.Header.Equals("Jahresgehalt")) {  
                e.Cancel = true;
            }            
        }

        

        private void datagrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e) {
            DataGridColumn col = e.Column;
            e.Cancel = false;
            if (col.Header.Equals("Lastname")) {
                TextBox t = e.EditingElement as TextBox;
                //DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;

            }
            if (col.Header.Equals("Firstname")) {
                TextBox t = e.EditingElement as TextBox;
                //DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;
            }
            if (col.Header.Equals("Monthsalary")) {
                TextBox t = e.EditingElement as TextBox;
                double monatsgehalt;
                if (double.TryParse(t.Text.Trim(), System.Globalization.NumberStyles.AllowDecimalPoint,
                           System.Globalization.CultureInfo.InvariantCulture, out monatsgehalt)) {
                    DataGridRow row = e.Row;
                    
                    ModelEmployee mit = (ModelEmployee)row.Item;
                    mit.Yearsalary = monatsgehalt * 12.0;
                    System.Diagnostics.Debug.WriteLine("datagrid_CellEditEnding: Jahresgehalt: " + mit.Yearsalary); 
                    mit.updateYearsalary();
                }
            }
            System.Diagnostics.Debug.WriteLine("datagrid_CellEditEnding");
        }



        private void datagrid_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (datagrid.SelectedIndex >= 0) {
                ModelEmployee mit  = empl_items[datagrid.SelectedIndex];
                LMitarbeiter.Content = mit.Lastname + ", " + mit.Firstname;
            }
        }

        private void bnTest_Click(object sender, RoutedEventArgs e) {
            ModelEmployee mit = empl_items[2];
            mit.Monthsalary = 2000;
            LMitarbeiter.Content = "Test: "+mit.Lastname + ", " + mit.Firstname;
            collView.Refresh();
        }



        


    }
}
