﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Reflection; // ICollectionView


using DB_ABT.UserControls;
using DB_ABT.DBS_Klassen;
using DB_ABT.ViewModels;
using DB_ABT.Models;
using DB_ABT.Fenster;


namespace DB_ABT.Views {
    /// <summary>
    /// Interaktionslogik für FMasterDetail.xaml
    /// </summary>
    public partial class FMasterDetail : Window, INavigatorbar {

        ObservableCollection_Employee class_Employee;
        ObservableCollection_Dept class_Dept;

        ObservableCollection<ModelEmployee> empl_items = new ObservableCollection<ModelEmployee>();
        ObservableCollection<ModelDept> dept_items = new ObservableCollection<ModelDept>();


        ICollectionView collViewDept = null; 
        ICollectionView collViewEmployee = null;


        public FMasterDetail(Window owner, ObservableCollection_Employee class_Employee, ObservableCollection_Dept class_Dept) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
            empl_items = class_Employee.Items;
            dept_items = class_Dept.Items;
            this.class_Employee = class_Employee;
            this.class_Dept = class_Dept;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            Title = "Department-Employee-Dialog";
            LEmployee.Content = "Employee: ";
            setDatabase_Work_Dept();
            setDatabase_Work_Employee();
            navibar1.setDatabaseParent(this, 1);
            navibar2.setDatabaseParent(this, 2);
        }


        private void setDatabase_Work_Dept() {
            this.datagridDept.AutoGenerateColumns = false;
            Type myType = typeof(ModelDept);
            PropertyInfo[] propertyInfos = myType.GetProperties(); // BindingFlags.Public
            foreach (PropertyInfo propertyInfo in propertyInfos) {
                System.Diagnostics.Debug.WriteLine(propertyInfo.Name);
                DataGridTextColumn dataColumn = new DataGridTextColumn();
                dataColumn.Header = propertyInfo.Name;

                Binding binding = new Binding(propertyInfo.Name);
                binding.Mode = BindingMode.TwoWay;
                binding.UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged;
                dataColumn.Binding = binding;
                datagridDept.Columns.Add(dataColumn);
            }
            datagridDept.ItemsSource = dept_items;
            setCountDept(dept_items.Count);

            collViewDept = CollectionViewSource.GetDefaultView(datagridDept.ItemsSource);
            collViewDept.SortDescriptions.Add(new SortDescription("ShortName", ListSortDirection.Ascending));            

            datagridDept.Columns[0].Width = 100;
            datagridDept.Columns[1].Width = 200; // datagridDept.Width - 200;
            datagridDept.Columns[2].Width = 50;
            datagridDept.CanUserAddRows = false;

            datagridDept.DataContext = dept_items[0];
        }


        private void setDatabase_Work_Employee() {
            this.datagridDept.AutoGenerateColumns = false;
            Type myType = typeof(ModelEmployee);
            PropertyInfo[] propertyInfos = myType.GetProperties(); // BindingFlags.Public
            foreach (PropertyInfo propertyInfo in propertyInfos) {
                if (propertyInfo.Name.Equals("Dept")) {
                }
                else {
                    System.Diagnostics.Debug.WriteLine(propertyInfo.Name);
                    DataGridTextColumn dataColumn = new DataGridTextColumn();
                    dataColumn.Header = propertyInfo.Name;

                    Binding binding = new Binding(propertyInfo.Name);
                    binding.Mode = BindingMode.TwoWay;
                    binding.UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged;  // neu

                    dataColumn.Binding = binding;

                    datagridEmployee.Columns.Add(dataColumn);
                }
            }
            datagridEmployee.ItemsSource = empl_items;
            setCountEmployee(empl_items.Count);

            collViewEmployee = CollectionViewSource.GetDefaultView(datagridEmployee.ItemsSource);
            collViewEmployee.SortDescriptions.Add(new SortDescription("Lastname", ListSortDirection.Ascending));
            collViewEmployee.SortDescriptions.Add(new SortDescription("Firstname", ListSortDirection.Ascending));
            collViewEmployee.Filter = EmployeeFilter;

            datagridEmployee.Columns[0].Width = 100;
            datagridEmployee.Columns[1].Width = 200; // datagridDept.Width - 200;
            datagridEmployee.Columns[2].Width = 50;
            datagridEmployee.CanUserAddRows = false;

            datagridEmployee.DataContext = empl_items[0];
        }

 

        private bool getModify() {
            bool modify = false;
            foreach (ModelDept dept in dept_items) {
                if (dept.modifyAttrib) modify = true;
                if (dept.newAttrib) modify = true;
            }
            if (class_Dept.getDeleteItemsCount() > 0) modify = true;
            

            foreach (ModelEmployee empl in empl_items) {
                if (empl.modifyAttrib) modify = true;
                if (empl.newAttrib) modify = true;
            }
            if (class_Employee.getDeleteItemsCount() > 0) modify = true;            

            return modify;
        }

        public MessageBoxResult readYesNoCancel(string message, string title) {
            return MessageBox.Show(message, title, MessageBoxButton.YesNoCancel, MessageBoxImage.Question);   // Exclamation
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = readYesNoCancel("Change the modification?", "Close Dialog");
                switch (res) {
                    case MessageBoxResult.Yes: save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                save2Database();
                class_Employee.clearModify();
                class_Dept.clearModify();                
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }




        // Spalten, die nicht editierbar sein sollen
        private void datagrid_BeginningEdit(object sender, DataGridBeginningEditEventArgs e) {
            DataGridColumn col = e.Column;
            if (col.Header.Equals("PIndex") || col.Header.Equals("Yearsalary")) {
                e.Cancel = true;
            }
        }

        private void datagrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e) {
            DataGridColumn col = e.Column;
            e.Cancel = false;
            if (col.Header.Equals("Lastname")) {
                TextBox t = e.EditingElement as TextBox;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;

            }
            if (col.Header.Equals("Firstname")) {
                TextBox t = e.EditingElement as TextBox;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;
            }
            if (col.Header.Equals("Monthsalary")) {                
                DataGridRow row = e.Row;
                ModelEmployee empl = (ModelEmployee)row.Item;
                empl.updateYearsalary();  // important
            }
            System.Diagnostics.Debug.WriteLine("datagrid_CellEditEnding");
        }


        bool tRowAction = false;
        private void tRow_TextChanged(object sender, TextChangedEventArgs e) {
            //if (!tRowAction) {
            //    int row;
            //    if (int.TryParse(tRow1.Text.Trim(), out row)) {
            //        if (row >= 0 && row < empl_items.Count) {
            //            datagridDept.SelectedIndex = row;
            //            datagrid_SelectionChanged(null, null);
            //            datagridDept.ScrollIntoView(datagridDept.SelectedItem);
            //            datagridDept.Focus();
            //        }
            //    }
            //}
            // tRow
        }


       

        private void setCountDept(long count) {
            //lRow1.Content = "von {" + count + "}";
        }

        private void setCountEmployee(long count) {
            //lRow2.Content = "von {" + count + "}";
        }

        private void save2Database() {
            class_Dept.save2Database();
            class_Employee.save2Database();
        }



        public void setFirst(int nr) {
            switch (nr) {
                case 1: datagridDept.SelectedIndex = 0;
                    datagridDept.ScrollIntoView(datagridDept.SelectedItem);
                    datagridDept.Focus();
                    break;
                case 2: datagridEmployee.SelectedIndex = 0;
                    datagridEmployee.ScrollIntoView(datagridEmployee.SelectedItem);
                    datagridEmployee.Focus();
                    break;
            }
        }

        public void setPrev(int nr) {
            switch (nr) {
                case 1: if (datagridDept.SelectedIndex > 0) {
                        datagridDept.SelectedIndex = datagridDept.SelectedIndex - 1;
                        datagridDept.ScrollIntoView(datagridDept.SelectedItem);
                        datagridDept.Focus();
                        }
                    break;
                case 2: if (datagridEmployee.SelectedIndex > 0) {
                    datagridEmployee.SelectedIndex = datagridEmployee.SelectedIndex - 1;
                    datagridEmployee.ScrollIntoView(datagridEmployee.SelectedItem);
                    datagridEmployee.Focus();
                    }
                    break;
            }
        }

        public void setNext(int nr) {
            switch (nr) {
                case 1: if (datagridDept.SelectedIndex < (dept_items.Count - 1)) {
                        datagridDept.SelectedIndex = datagridDept.SelectedIndex + 1;
                        datagridDept.ScrollIntoView(datagridDept.SelectedItem);
                        datagridDept.Focus();
                        }
                    break;
                case 2: if (datagridEmployee.SelectedIndex < (empl_items.Count - 1)) {
                    datagridEmployee.SelectedIndex = datagridEmployee.SelectedIndex + 1;
                    datagridEmployee.ScrollIntoView(datagridEmployee.SelectedItem);
                    datagridEmployee.Focus();
                    }
                    break;
            }
        }

        public void setLast(int nr) {
            switch (nr) {
                case 1: datagridDept.SelectedIndex = dept_items.Count - 1;
                        datagridDept.ScrollIntoView(datagridDept.SelectedItem);
                        datagridDept.Focus();
                    break;
                case 2: datagridEmployee.SelectedIndex = empl_items.Count - 1;
                    datagridEmployee.ScrollIntoView(datagridEmployee.SelectedItem);
                    datagridEmployee.Focus();

                    break;
            }
        }


        public void setInsert(int nr) {
            switch (nr) {
                case 1: 
                    break;
                case 2: ModelEmployee empl = new ModelEmployee();
                    FReadInputMitarbeiter dialog1 = new FReadInputMitarbeiter(this, empl);
                    dialog1.ShowDialog();
                    if (dialog1.Retcode) {
                        empl.Firstname = dialog1.Firstname;
                        empl.Lastname = dialog1.Lastname;
                        empl.Monthsalary = dialog1.Monthsalary;
                        empl.DeptIindex = dialog1.AIndex;
                        Dbs dbs = Dbs.getDbsInstance();
                        empl.PIndex = Dbs.getSequenz("seq");
                        empl_items.Add(empl);
                        navibar1.setCount(empl_items.Count);
                    }
                    break;
            }
        }

        public void setDelete(int nr) {
            switch (nr) {
                case 1: ModelDept dept = (ModelDept)datagridDept.SelectedItem;
                    if (dept != null) {
                        // check, ob die Abteilung noch benutzt wird?
                        bool gefunden = Dbs_QueriesExceute.isForeignKeyInMitarbeiter(dept.PIndex);
                        if (gefunden) {
                            Basis.Message("Diese Abteilung darf nicht gelöscht werden, da sie noch verwendet wird", "Hinweis");
                        }
                        else {
                            if (Basis.readYesNo("Wollen Sie wirklich diese Abteilung löschen ?\r\n" +
                                "Abteilung: " + dept.ShortName + " (" + dept.LongName + ")", "Abteilung löschen")) {
                                class_Dept.Delete(dept);
                                navibar1.setCount(dept_items.Count);
                                // setFilter(filter);
                            }
                        }
                    }
                    break;
                case 2: ModelEmployee empl = (ModelEmployee)datagridEmployee.SelectedItem;
                    if (empl != null) {
                        if (Basis.readYesNo("Wollen Sie wirklich diesen Mitarbeiter löschen ?\r\n" +
                                "Mitarbeiter: " + empl.Lastname + ", " + empl.Firstname + ")", "Mitarbeiter löschen")) {
                            class_Employee.Delete(empl);
                            navibar1.setCount(empl_items.Count);
                        }
                    }
                    break;
            }
        }

        public void setCancel(int nr) {
            switch (nr) {
                case 1:
                    break;
                case 2:
                    break;
            }
            //((DataRowView)datagrid.SelectedItem).Row.CancelEdit();
        }



        private void datagridDept_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (datagridDept.SelectedIndex >= 0) {
                ModelDept dept = (ModelDept)datagridDept.SelectedItem;
                LDepartment.Content = dept.ShortName + ", " + dept.LongName;
                tRowAction = true;
                navibar1.setRow(datagridDept.SelectedIndex);
                tRowAction = false;
                department_employee_aindex = dept.PIndex;
                setFilter(2, "");
            }
        }

        private void datagridEmployee_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (datagridEmployee.SelectedIndex >= 0) {
                navibar2.setRow(datagridEmployee.SelectedIndex);
            }
        }
               



        public void setFilter(int nr, String filter) {
            String f = filter.Trim();
            switch (nr) {
                case 1: if (f.Length == 0) {
                    collViewDept.Filter = null;
                    }
                    else {
                        collViewDept.Filter = dept => (dept as ModelDept).ShortName.StartsWith(filter);
                    }
                    navibar1.setCount(datagridDept.Items.Count);
                    break;
                case 2: filterEmployeeString = f;
                        collViewEmployee.Refresh();
                        navibar1.setCount(datagridEmployee.Items.Count);
                    break;
            }
        }

        // ------------------------------------------------------------------
        String filterEmployeeString = "";
        private long department_employee_aindex = 0;

        private bool EmployeeFilter(object item) {
            ModelEmployee empl = item as ModelEmployee;
            if (filterEmployeeString.Length == 0 && department_employee_aindex == 0)
                return true;
            else
                if (filterEmployeeString.Length > 0 && department_employee_aindex > 0)
                    return empl.Lastname.StartsWith(filterEmployeeString) && empl.DeptIindex == department_employee_aindex;
                else
                    if (filterEmployeeString.Length > 0)
                        return empl.Lastname.StartsWith(filterEmployeeString);
                    else
                        if (department_employee_aindex > 0)
                            return empl.DeptIindex == department_employee_aindex;
                        else
                            return true;
        } // EmployeeFilter


        private void tFilter2_TextChanged(object sender, TextChangedEventArgs e) {
            //
        }


       

        private long sequence = 0;
        private long getSequenz(String seqname) {
            return ++sequence;
        }





        




    }
}
